@@ -38,7 +38,7 @@
 #endif
 
 /* Raptor includes */
-#include "raptor.h"
+#include "raptor2.h"
 #include "raptor_internal.h"
 #ifdef RAPTOR_NFC_CHECK
 #include "raptor_nfc.h"
@@ -212,7 +212,8 @@ raptor_unicode_utf8_string_get_char(const unsigned char *input, size_t length,
       c |= in;
       /* FALLTHROUGH */
     case 2:
-      in=*input++ & 0x3f;
+      /* '*input' used here since we never need to use new value of input [CLANG] */
+      in = *input & 0x3f;
       c= c << 6;
       c |= in;
       /* FALLTHROUGH */
