@@ -467,6 +467,32 @@ raptor_www_set_uri_filter(raptor_www* www,
 }
 
 
+/**
+ * raptor_www_set_ssl_cert_options:
+ * @www: WWW object
+ * @cert_filename: SSL client certificate file
+ * @cert_type: SSL client certificate type (default is "PEM")
+ * @cert_passphrase: SSL client certificate password
+ * 
+ * Set SSL client certificate options (where supported)
+ *
+ * Return value: non-0 when setting options is not supported
+ **/
+int
+raptor_www_set_ssl_cert_options(raptor_www* www,
+                                const char* cert_filename,
+                                const char* cert_type,
+                                const char* cert_passphrase)
+{
+#ifdef RAPTOR_WWW_LIBCURL
+  return raptor_www_curl_set_ssl_cert_options(www, cert_filename, cert_type,
+                                              cert_passphrase);
+#else
+  return 1;
+#endif
+}
+
+
 /**
  * raptor_www_get_connection:
  * @www: #raptor_www object 
@@ -539,16 +565,14 @@ raptor_www_error(raptor_www* www, const char *message, ...)
 static int 
 raptor_www_file_handle_fetch(raptor_www* www, FILE* fh) 
 {
-  unsigned char buffer[RAPTOR_WWW_BUFFER_SIZE+1];
-  
   while(!feof(fh)) {
-    int len = fread(buffer, 1, RAPTOR_WWW_BUFFER_SIZE, fh);
+    size_t len = fread(www->buffer, 1, RAPTOR_WWW_BUFFER_SIZE, fh);
     if(len > 0) {
       www->total_bytes += len;
-      buffer[len]='\0';
+      www->buffer[len]='\0';
       
       if(www->write_bytes)
-        www->write_bytes(www, www->write_bytes_userdata, buffer, len, 1);
+        www->write_bytes(www, www->write_bytes_userdata, www->buffer, len, 1);
     }
 
     if(feof(fh) || www->failed)
@@ -672,7 +696,7 @@ raptor_www_fetch_to_string_write_bytes(raptor_www* www, void *userdata,
                                        size_t nmemb)
 {
   raptor_stringbuffer* sb = (raptor_stringbuffer*)userdata;
-  int len = size*nmemb;
+  size_t len = size * nmemb;
 
   raptor_stringbuffer_append_counted_string(sb, (unsigned char*)ptr, len, 1);
 }
